@@ -1,8 +1,7 @@
 # et:ts=4
 # portmpkg.tcl
-# $Id: portmpkg.tcl 101268 2013-01-07 14:18:10Z jmr@macports.org $
 #
-# Copyright (c) 2005, 2007 - 2012 The MacPorts Project
+# Copyright (c) 2005, 2007 - 2013 The MacPorts Project
 # Copyright (c) 2002 - 2004 Apple Inc.
 # All rights reserved.
 #
@@ -17,7 +16,7 @@
 # 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -48,63 +47,62 @@ options package.destpath package.flat
 set_ui_prefix
 
 proc portmpkg::mpkg_main {args} {
-    global subport version revision os.major package.destpath package.flat UI_PREFIX
+    global subport epoch version revision os.major package.destpath package.flat UI_PREFIX
 
     if {!${package.flat} || ${os.major} < 10} {
         # Make sure the destination path exists.
         file mkdir ${package.destpath}
     }
 
-    return [package_mpkg $subport $version $revision]
+    return [package_mpkg $subport $epoch $version $revision]
 }
 
 proc portmpkg::make_dependency_list {portname destination} {
     global requested_variations prefix package.destpath package.flat
-	set result {}
-	if {[catch {set res [mport_lookup $portname]} error]} {
-		global errorInfo
-		ui_debug "$errorInfo"
-		return -code error "port lookup failed: $error"
-	}
-	array set portinfo [lindex $res 1]
+    set result [list]
+    if {[catch {set res [mport_lookup $portname]} error]} {
+        ui_debug $::errorInfo
+        return -code error "port lookup failed: $error"
+    }
+    array set portinfo [lindex $res 1]
 
-	if {[getuid] == 0 && [geteuid] != 0} {
-		setegid 0; seteuid 0
-		set deprivileged 1
-	}
+    if {[getuid] == 0 && [geteuid] != 0} {
+        seteuid 0; setegid 0
+        set deprivileged 1
+    }
 
-	set mport [mport_open $portinfo(porturl) [list prefix $prefix package.destpath ${destination} package.flat ${package.flat} subport $portinfo(name)] [array get requested_variations]]
+    set mport [mport_open $portinfo(porturl) [list prefix $prefix package.destpath ${destination} package.flat ${package.flat} subport $portinfo(name)] [array get requested_variations]]
 
     if {[info exists deprivileged]} {
-	    global macportsuser
-		setegid [uname_to_gid "$macportsuser"]
-		seteuid [name_to_uid "$macportsuser"]
-	}
+        global macportsuser
+        setegid [uname_to_gid "$macportsuser"]
+        seteuid [name_to_uid "$macportsuser"]
+    }
 
     unset portinfo
     array set portinfo [mport_info $mport]
 
     # get the union of depends_run and depends_lib
-    set depends {}
-    if {[info exists portinfo(depends_run)]} { eval "lappend depends $portinfo(depends_run)" }
-    if {[info exists portinfo(depends_lib)]} { eval "lappend depends $portinfo(depends_lib)" }
+    set depends [list]
+    if {[info exists portinfo(depends_run)]} { lappend depends {*}$portinfo(depends_run) }
+    if {[info exists portinfo(depends_lib)]} { lappend depends {*}$portinfo(depends_lib) }
 
     foreach depspec $depends {
         set dep [_get_dep_port $depspec]
-        if {$dep != ""} {
-            eval "lappend result [make_dependency_list $dep $destination]"
+        if {$dep ne ""} {
+            lappend result {*}[make_dependency_list $dep $destination]
         }
     }
 
-    lappend result [list $portinfo(name) $portinfo(version) $mport]
-	return $result
+    lappend result [list $portinfo(name) $portinfo(version) $portinfo(revision) $mport]
+    return $result
 }
 
-proc portmpkg::make_one_package {portname portversion mport} {
-	if {[getuid] == 0 && [geteuid] != 0} {
-		setegid 0; seteuid 0
-		set deprivileged 1
-	}
+proc portmpkg::make_one_package {portname mport} {
+    if {[getuid] == 0 && [geteuid] != 0} {
+        seteuid 0; setegid 0
+        set deprivileged 1
+    }
 
     ui_debug "building dependency package: $portname"
     set result [mport_exec $mport pkg]
@@ -113,56 +111,62 @@ proc portmpkg::make_one_package {portname portversion mport} {
         error "Processing of port $portname failed"
     }
 
-	if {[info exists deprivileged]} {
-	    global macportsuser
-		setegid [uname_to_gid "$macportsuser"]
-		seteuid [name_to_uid "$macportsuser"]
-	}
+    if {[info exists deprivileged]} {
+        global macportsuser
+        setegid [uname_to_gid "$macportsuser"]
+        seteuid [name_to_uid "$macportsuser"]
+    }
 }
 
-proc portmpkg::package_mpkg {portname portversion portrevision} {
-    global portdbpath os.major destpath workpath prefix porturl description package.destpath package.flat long_description homepage depends_run depends_lib
+proc portmpkg::mpkg_path {portname portversion portrevision} {
+    global package.destpath
+    return "${package.destpath}/[portpkg::image_name ${portname} ${portversion} ${portrevision}].mpkg"
+}
 
-	set mpkgpath ${package.destpath}/${portname}-${portversion}.mpkg
+proc portmpkg::package_mpkg {portname portepoch portversion portrevision} {
+    global os.major destpath workpath prefix porturl description package.destpath package.flat long_description homepage
+
+    set mpkgpath [portmpkg::mpkg_path $portname $portversion $portrevision]
 
     if {${package.flat} && ${os.major} >= 10} {
-        set pkgpath ${package.destpath}/${portname}-${portversion}-component.pkg
+        set pkgpath ${package.destpath}/[portpkg::image_name ${portname} ${portversion} ${portrevision}]-component.pkg
         set packages_path ${workpath}/mpkg_packages
         set resources_path ${workpath}/mpkg_resources
     } else {
-        set pkgpath ${package.destpath}/${portname}-${portversion}.pkg
-	    set packages_path ${mpkgpath}/Contents/Packages
-	    set resources_path ${mpkgpath}/Contents/Resources
-	}
-	system "mkdir -p -m 0755 ${packages_path}"
-	system "mkdir -p -m 0755 ${resources_path}"
+        set pkgpath ${package.destpath}/[portpkg::image_name ${portname} ${portversion} ${portrevision}].pkg
+        set packages_path ${mpkgpath}/Contents/Packages
+        set resources_path ${mpkgpath}/Contents/Resources
+    }
+    system "mkdir -p -m 0755 [shellescape ${packages_path}]"
+    system "mkdir -p -m 0755 [shellescape ${resources_path}]"
 
-	set dependencies {}
-	# get deplist
-	set deps [make_dependency_list $portname $packages_path]
-	set deps [lsort -unique $deps]
-	foreach dep $deps {
-		set name [lindex $dep 0]
-		set vers [lindex $dep 1]
-		set mport [lindex $dep 2]
-		# don't re-package ourself
-		if {$name != $portname} {
-			make_one_package $name $vers $mport
-			if {${package.flat} && ${os.major} >= 10} {
-			    lappend dependencies org.macports.${name} ${name}-${vers}-component.pkg
-			} else {
-			    lappend dependencies ${name}-${vers}.pkg
-			}
-		}
-	}
-	if {${package.flat} && ${os.major} >= 10} {
-	    lappend dependencies org.macports.${portname} ${portname}-${portversion}-component.pkg
-	} else {
-	    lappend dependencies ${portname}-${portversion}.pkg
-	}
+    set dependencies [list]
+    # get deplist
+    set deps [make_dependency_list $portname $packages_path]
+    set deps [lsort -unique $deps]
+    foreach dep $deps {
+        set name [lindex $dep 0]
+        set vers [lindex $dep 1]
+        set rev [lindex $dep 2]
+        set mport [lindex $dep 3]
+        # don't re-package ourself
+        if {$name ne $portname} {
+            make_one_package $name $mport
+            if {${package.flat} && ${os.major} >= 10} {
+                lappend dependencies org.macports.${name} [portpkg::image_name ${name} ${vers} ${rev}]-component.pkg
+            } else {
+                lappend dependencies [portpkg::image_name ${name} ${vers} ${rev}].pkg
+            }
+        }
+    }
+    if {${package.flat} && ${os.major} >= 10} {
+        lappend dependencies org.macports.${portname} [portpkg::image_name ${portname} ${portversion} ${portrevision}]-component.pkg
+    } else {
+        lappend dependencies [portpkg::image_name ${portname} ${portversion} ${portrevision}].pkg
+    }
 
     # copy our own pkg into the mpkg
-    system "cp -PR ${pkgpath} ${packages_path}"
+    system "cp -PR [shellescape ${pkgpath}] [shellescape ${packages_path}]"
 
     if {!${package.flat} || ${os.major} < 10} {
         portpkg::write_PkgInfo ${mpkgpath}/Contents/PkgInfo
@@ -178,18 +182,19 @@ proc portmpkg::package_mpkg {portname portversion portrevision} {
             set pkg_$variable [set $variable]
         }
     }
-    portpkg::write_welcome_html ${resources_path}/Welcome.html $portname $portversion $pkg_long_description $pkg_description $pkg_homepage
+    portpkg::write_welcome_html ${resources_path}/Welcome.html $portname $portversion $portrevision $pkg_long_description $pkg_description $pkg_homepage
     file copy -force -- [getportresourcepath $porturl "port1.0/package/background.tiff"] ${resources_path}/background.tiff
 
     if {${package.flat} && ${os.major} >= 10} {
         write_distribution ${workpath}/Distribution $portname $dependencies
         set productbuild [findBinary productbuild]
-        set cmdline "$productbuild --resources ${resources_path} --identifier org.macports.mpkg.${portname} --distribution ${workpath}/Distribution --package-path ${packages_path} ${mpkgpath}"
+        set v [portpkg::mp_version_to_apple_version $portepoch $portversion $portrevision]
+        set cmdline "$productbuild --resources [shellescape ${resources_path}] --identifier org.macports.mpkg.${portname} --distribution [shellescape ${workpath}/Distribution] --package-path [shellescape ${packages_path}] --version ${v} [shellescape ${mpkgpath}]"
         ui_debug "Running command line: $cmdline"
         system $cmdline
     }
 
-	return 0
+    return 0
 }
 
 proc portmpkg::write_distribution {dfile portname dependencies} {
@@ -226,61 +231,61 @@ proc portmpkg::write_distribution {dfile portname dependencies} {
 }
 
 proc portmpkg::xml_escape {s} {
-	regsub -all {&} $s {\&amp;} s
-	regsub -all {<} $s {\&lt;} s
-	regsub -all {>} $s {\&gt;} s
-	return $s
+    regsub -all {&} $s {\&amp;} s
+    regsub -all {<} $s {\&lt;} s
+    regsub -all {>} $s {\&gt;} s
+    return $s
 }
 
 proc portmpkg::mpkg_write_info_plist {infofile portname portversion portrevision destination dependencies} {
-	set vers [split $portversion "."]
-	
-	if {[string index $destination end] != "/"} {
-		append destination /
-	}
-	
-	set depxml ""
-	foreach dep $dependencies {
-		set dep [xml_escape $dep]
-		append depxml "<dict>
-			<key>IFPkgFlagPackageLocation</key>
-			<string>${dep}</string>
-			<key>IFPkgFlagPackageSelection</key>
-			<string>selected</string>
-		</dict>
-		"
-	}
+    set vers [split $portversion "."]
 
-	set portname [xml_escape $portname]
-	set portversion [xml_escape $portversion]
-	set portrevision [xml_escape $portrevision]
+    if {[string index $destination end] ne "/"} {
+        append destination /
+    }
 
-	set infofd [open ${infofile} w+]
-	puts $infofd {<?xml version="1.0" encoding="UTF-8"?>
+    set depxml ""
+    foreach dep $dependencies {
+        set dep [xml_escape $dep]
+        append depxml "<dict>
+            <key>IFPkgFlagPackageLocation</key>
+            <string>${dep}</string>
+            <key>IFPkgFlagPackageSelection</key>
+            <string>selected</string>
+        </dict>
+        "
+    }
+
+    set portname [xml_escape $portname]
+    set portversion [xml_escape $portversion]
+    set portrevision [xml_escape $portrevision]
+
+    set infofd [open ${infofile} w+]
+    puts $infofd {<?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
 <plist version="1.0">
 }
-	puts $infofd "<dict>
-	<key>CFBundleGetInfoString</key>
-	<string>${portname} ${portversion}</string>
-	<key>CFBundleIdentifier</key>
-	<string>org.macports.mpkg.${portname}</string>
-	<key>CFBundleName</key>
-	<string>${portname}</string>
-	<key>CFBundleShortVersionString</key>
-	<string>${portversion}</string>
-	<key>IFMajorVersion</key>
-	<integer>${portrevision}</integer>
-	<key>IFMinorVersion</key>
-	<integer>0</integer>
-	<key>IFPkgFlagComponentDirectory</key>
-	<string>./Contents/Packages</string>
-	<key>IFPkgFlagPackageList</key>
-	<array>
-		${depxml}</array>
-	<key>IFPkgFormatVersion</key>
-	<real>0.10000000149011612</real>
+    puts $infofd "<dict>
+    <key>CFBundleGetInfoString</key>
+    <string>${portname} ${portversion}</string>
+    <key>CFBundleIdentifier</key>
+    <string>org.macports.mpkg.${portname}</string>
+    <key>CFBundleName</key>
+    <string>${portname}</string>
+    <key>CFBundleShortVersionString</key>
+    <string>${portversion}</string>
+    <key>IFMajorVersion</key>
+    <integer>${portrevision}</integer>
+    <key>IFMinorVersion</key>
+    <integer>0</integer>
+    <key>IFPkgFlagComponentDirectory</key>
+    <string>./Contents/Packages</string>
+    <key>IFPkgFlagPackageList</key>
+    <array>
+        ${depxml}</array>
+    <key>IFPkgFormatVersion</key>
+    <real>0.10000000149011612</real>
 </dict>
 </plist>"
-	close $infofd
+    close $infofd
 }
