@@ -47,9 +47,9 @@ namespace eval portfetch::mirror_sites {
 # percent-encode all characters in str that are not unreserved in URIs
 proc portfetch::percent_encode {str} {
     set outstr ""
-    while {$str ne ""} {
-        set char [string index $str 0]
-        set str [string range $str 1 end]
+    set len [string length $str]
+    for {set i 0} {$i < $len} {incr i} {
+        set char [string index $str $i]
         switch -- $char {
             {-} -
             {.} -
@@ -93,7 +93,7 @@ proc portfetch::mirror_sites {mirrors tag subdir mirrorfile} {
         if {$mirrors ne $global_mirror_site} {
             ui_warn "[format [msgcat::mc "No mirror sites on file for class %s"] $mirrors]"
         }
-        return {}
+        return [list]
     }
 
     set ret [list]
@@ -247,7 +247,7 @@ proc portfetch::sortsites {urls default_listvar} {
             seteuid 0; setegid 0
         }
 
-        set hosts {}
+        set hosts [list]
         foreach site $urllist {
             if {[string range $site 0 6] eq "file://"} {
                 set pingtimes(localhost) 0
