@@ -32,6 +32,7 @@
 
 package provide portbuild 1.0
 package require portutil 1.0
+package require portprogress 1.0
 
 set org.macports.build [target_new org.macports.build portbuild::build_main]
 target_provides ${org.macports.build} build
@@ -44,6 +45,7 @@ namespace eval portbuild {
 # define options
 options build.asroot \
         build.jobs \
+        build.jobs_arg \
         build.target \
         use_parallel_build
 commands build
@@ -53,6 +55,7 @@ default build.dir {${worksrcpath}}
 default build.cmd {[portbuild::build_getmaketype]}
 default build.nice {${buildnicevalue}}
 default build.jobs {[portbuild::build_getjobs]}
+default build.jobs_arg {[portbuild::build_getjobsarg]}
 default build.pre_args {[portbuild::build_getargs]}
 default build.target "all"
 default build.type "default"
@@ -86,67 +89,78 @@ options build.type.add_deps
 default build.type.add_deps yes
 
 proc portbuild::build_getmaketype {args} {
-    if {[option build.type] eq "default"} {
-        return [findBinary make $portutil::autoconf::make_path]
-    }
-    switch -exact -- [option build.type] {
-        bsd {
-            if {[option os.platform] eq "darwin"} {
-                return [findBinary bsdmake $portutil::autoconf::bsdmake_path]
-            } elseif {[option os.platform] eq "freebsd"} {
-                return [findBinary make $portutil::autoconf::make_path]
-            } else {
-                return [findBinary pmake $portutil::autoconf::bsdmake_path]
-            }
+    macports_try -pass_signal {
+        if {[option build.type] eq "default"} {
+            return [findBinary make $portutil::autoconf::make_path]
         }
-        gnu {
-            if {[option os.platform] eq "darwin"} {
-                return [findBinary gnumake $portutil::autoconf::gnumake_path]
-            } elseif {[option os.platform] eq "linux"} {
-                return [findBinary make $portutil::autoconf::make_path]
-            } else {
-                return [findBinary gmake $portutil::autoconf::gnumake_path]
+        switch -exact -- [option build.type] {
+            bsd {
+                if {[option os.platform] eq "darwin"} {
+                    return [findBinary bsdmake $portutil::autoconf::bsdmake_path]
+                } elseif {[option os.platform] eq "freebsd"} {
+                    return [findBinary make $portutil::autoconf::make_path]
+                } else {
+                    return [findBinary pmake $portutil::autoconf::bsdmake_path]
+                }
             }
-        }
-        pbx -
-        xcode {
-            if {[option os.platform] ne "darwin"} {
-                return -code error "[format [msgcat::mc "This port requires 'xcodebuild', which is not available on %s."] [option os.platform]]"
+            gnu {
+                if {[option os.platform] eq "darwin"} {
+                    return [findBinary gnumake $portutil::autoconf::gnumake_path]
+                } elseif {[option os.platform] eq "linux"} {
+                    return [findBinary make $portutil::autoconf::make_path]
+                } else {
+                    return [findBinary gmake $portutil::autoconf::gnumake_path]
+                }
             }
+            pbx -
+            xcode {
+                if {[option os.platform] ne "darwin"} {
+                    error "[format [msgcat::mc "This port requires 'xcodebuild', which is not available on %s."] [option os.platform]]"
+                }
 
-            global xcodebuildcmd
-            if {$xcodebuildcmd ne "none"} {
-                return $xcodebuildcmd
-            } else {
-                return -code error "xcodebuild was not found on this system!"
+                global xcodebuildcmd
+                if {$xcodebuildcmd ne "none"} {
+                    return $xcodebuildcmd
+                } else {
+                    error "xcodebuild was not found on this system!"
+                }
+            }
+            default {
+                ui_warn "[format [msgcat::mc "Unknown build.type %s, using 'gnumake'"] [option build.type]]"
+                return [findBinary gnumake $portutil::autoconf::gnumake_path]
             }
         }
-        default {
-            ui_warn "[format [msgcat::mc "Unknown build.type %s, using 'gnumake'"] [option build.type]]"
-            return [findBinary gnumake $portutil::autoconf::gnumake_path]
-        }
+    } on error {eMessage} {
+        ui_warn $eMessage
+        ui_warn "Unable to find build command for build.type '[option build.type]'"
     }
+    return ""
 }
 
 proc portbuild::build_getjobs {args} {
-    global buildmakejobs
+    global buildmakejobs use_parallel_build
     set jobs $buildmakejobs
+    # If parallel disabled disabled, return 1
+    if {![tbool use_parallel_build]} {
+        ui_debug "port disallows a parallel build, setting build jobs to 1"
+        set jobs 1
+    }
     # if set to '0', use the number of cores for the number of jobs
     if {$jobs == 0} {
-        try -pass_signal {
+        macports_try -pass_signal {
             set jobs [sysctl hw.activecpu]
-        } catch {{*} eCode eMessage} {
+        } on error {} {
+            set jobs 2
             ui_warn "failed to determine the number of available CPUs (probably not supported on this platform)"
             ui_warn "defaulting to $jobs jobs, consider setting buildmakejobs to a nonzero value in macports.conf"
-            set jobs 2
         }
 
-        try -pass_signal {
+        macports_try -pass_signal {
             set memsize [sysctl hw.memsize]
             if {$jobs > $memsize / (1024 * 1024 * 1024) + 1} {
                 set jobs [expr {$memsize / (1024 * 1024 * 1024) + 1}]
             }
-        } catch {*} {}
+        } on error {} {}
     }
     if {![string is integer -strict $jobs] || $jobs <= 1} {
         set jobs 1
@@ -166,18 +180,16 @@ proc portbuild::build_getargs {args} {
 }
 
 proc portbuild::build_getjobsarg {args} {
-    # check if port allows a parallel build
-    global use_parallel_build
-    if {![tbool use_parallel_build]} {
-        ui_debug "port disallows a parallel build"
+    set cmdname [file tail [lindex [option build.cmd] 0]]
+    if {![exists build.jobs] || \
+            !([string match "*make" $cmdname] || \
+              "ninja" eq $cmdname || \
+              "scons" eq $cmdname)} {
         return ""
     }
 
-    if {![exists build.jobs] || !([string match "*make*" [option build.cmd]] || [string match "*scons*" [option build.cmd]])} {
-        return ""
-    }
     set jobs [option build.jobs]
-    if {![string is integer -strict $jobs] || $jobs <= 1} {
+    if {![string is integer -strict $jobs] || $jobs < 1 || ($cmdname ne "ninja" && $jobs < 2)} {
         return ""
     }
     return " -j$jobs"
@@ -190,13 +202,15 @@ proc portbuild::build_start {args} {
 }
 
 proc portbuild::build_main {args} {
-    global build.cmd
+    global build.cmd build.jobs_arg
 
-    set jobs_suffix [build_getjobsarg]
+    if {${build.cmd} eq ""} {
+        error "No build command found"
+    }
 
     set realcmd ${build.cmd}
-    set build.cmd "${build.cmd}$jobs_suffix"
-    command_exec build
+    append build.cmd ${build.jobs_arg}
+    command_exec -callback portprogress::target_progress_callback build
     set build.cmd ${realcmd}
     return 0
 }
