@@ -1,9 +1,8 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
-# portclean.tcl
-# $Id: portclean.tcl 72989 2010-10-31 19:53:25Z jmr@macports.org $
 #
+# Copyright (c) 2005-2007, 2009-2011, 2013 The MacPorts Project
 # Copyright (c) 2004 Robert Shaw <rshaw@opendarwin.org>
-# Copyright (c) 2002 - 2003 Apple Computer, Inc.
+# Copyright (c) 2002 - 2003 Apple Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -14,7 +13,7 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
+# 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
 #
@@ -52,37 +51,36 @@ set_ui_prefix
 proc portclean::clean_start {args} {
     global UI_PREFIX
 
-    ui_msg "$UI_PREFIX [format [msgcat::mc "Cleaning %s"] [option name]]"
+    ui_notice "$UI_PREFIX [format [msgcat::mc "Cleaning %s"] [option subport]]"
+
+    if {[getuid] == 0 && [geteuid] != 0} {
+        elevateToRoot "clean"
+    }
 }
 
 proc portclean::clean_main {args} {
-    global UI_PREFIX
-    global ports_clean_dist ports_clean_work ports_clean_archive ports_clean_logs
-    global ports_clean_all keeplogs usealtworkpath
+    global UI_PREFIX ports_clean_dist ports_clean_work ports_clean_logs \
+           ports_clean_archive ports_clean_all keeplogs
 
-    if {$usealtworkpath} {
-        ui_warn "Only cleaning in ~/.macports; insufficient privileges for standard locations"
-    }
-
-    if {[info exists ports_clean_all] && $ports_clean_all == "yes" || \
-        [info exists ports_clean_dist] && $ports_clean_dist == "yes"} {
-        ui_info "$UI_PREFIX [format [msgcat::mc "Removing distfiles for %s"] [option name]]"
+    if {[info exists ports_clean_all] && $ports_clean_all eq "yes" || \
+        [info exists ports_clean_dist] && $ports_clean_dist eq "yes"} {
+        ui_info "$UI_PREFIX [format [msgcat::mc "Removing distfiles for %s"] [option subport]]"
         clean_dist
     }
-    if {([info exists ports_clean_all] && $ports_clean_all == "yes" || \
-        [info exists ports_clean_archive] && $ports_clean_archive == "yes")
-        && !$usealtworkpath} {
-        ui_info "$UI_PREFIX [format [msgcat::mc "Removing archives for %s"] [option name]]"
+    if {[info exists ports_clean_all] && $ports_clean_all eq "yes" || \
+        [info exists ports_clean_archive] && $ports_clean_archive eq "yes"} {
+        ui_info "$UI_PREFIX [format [msgcat::mc "Removing temporary archives for %s"] [option subport]]"
         clean_archive
     }
-    if {[info exists ports_clean_all] && $ports_clean_all == "yes" || \
-        [info exists ports_clean_work] && $ports_clean_work == "yes" || \
-        (!([info exists ports_clean_archive] && $ports_clean_archive == "yes"))} {
-         ui_info "$UI_PREFIX [format [msgcat::mc "Removing work directory for %s"] [option name]]"
+    if {[info exists ports_clean_all] && $ports_clean_all eq "yes" || \
+        [info exists ports_clean_work] && $ports_clean_work eq "yes" || \
+        [info exists ports_clean_archive] && $ports_clean_archive eq "yes" || \
+        [info exists ports_clean_dist] && $ports_clean_dist eq "yes" || \
+        !([info exists ports_clean_logs] && $ports_clean_logs eq "yes")} {
+         ui_info "$UI_PREFIX [format [msgcat::mc "Removing work directory for %s"] [option subport]]"
          clean_work
     }
-    if {(([info exists ports_clean_logs] && $ports_clean_logs == "yes") || ($keeplogs == "no"))
-        && !$usealtworkpath} {
+    if {([info exists ports_clean_logs] && $ports_clean_logs eq "yes") || ($keeplogs eq "no")} {
         clean_logs
     }
 
@@ -94,7 +92,7 @@ proc portclean::clean_main {args} {
 # This is crude, but works.
 #
 proc portclean::clean_dist {args} {
-    global ports_force name distpath dist_subdir distfiles patchfiles usealtworkpath portdbpath altprefix
+    global name ports_force distpath dist_subdir distfiles patchfiles portdbpath
 
     # remove known distfiles for sure (if they exist)
     set count 0
@@ -110,14 +108,6 @@ proc portclean::clean_dist {args} {
             }
             incr count
         }
-        if {!$usealtworkpath && [file isfile ${altprefix}${distfile}]} {
-            ui_debug "Removing file: ${altprefix}${distfile}"
-            if {[catch {delete ${altprefix}${distfile}} result]} {
-                ui_debug "$::errorInfo"
-                ui_error "$result"
-            }
-            incr count
-        }
     }
     if {$count > 0} {
         ui_debug "$count distfile(s) removed."
@@ -127,7 +117,7 @@ proc portclean::clean_dist {args} {
 
     set count 0
     if {![info exists patchfiles]} {
-        set patchfiles ""
+        set patchfiles [list]
     }
     foreach file $patchfiles {
         set patchfile [getdistname $file]
@@ -141,14 +131,6 @@ proc portclean::clean_dist {args} {
             }
             incr count
         }
-        if {!$usealtworkpath && [file isfile ${altprefix}${patchfile}]} {
-            ui_debug "Removing file: ${altprefix}${patchfile}"
-            if {[catch {delete ${altprefix}${patchfile}} result]} {
-                ui_debug "$::errorInfo"
-                ui_error "$result"
-            }
-            incr count
-        }
     }
     if {$count > 0} {
         ui_debug "$count patchfile(s) removed."
@@ -159,8 +141,8 @@ proc portclean::clean_dist {args} {
     # next remove dist_subdir if only needed for this port,
     # or if user forces us to
     set dirlist [list]
-    if {$dist_subdir != $name} {
-        if {!([info exists ports_force] && $ports_force == "yes")
+    if {$dist_subdir ne $name} {
+        if {!([info exists ports_force] && $ports_force eq "yes")
             && [file isdirectory $distpath]
             && [llength [readdir $distpath]] > 0} {
             ui_warn [format [msgcat::mc "Distfiles directory '%s' may contain distfiles needed for other ports, use the -f flag to force removal" ] $distpath]
@@ -174,11 +156,7 @@ proc portclean::clean_dist {args} {
     # loop through directories
     set count 0
     foreach dir $dirlist {
-        if {$usealtworkpath} {
-            set distdir [file join ${altprefix}${portdbpath} distfiles $dir]
-        } else {
-            set distdir [file join ${portdbpath} distfiles $dir]
-        }
+        set distdir [file join ${portdbpath} distfiles $dir]
         if {[file isdirectory $distdir]} {
             ui_debug "Removing directory: ${distdir}"
             if {[catch {delete $distdir} result]} {
@@ -187,14 +165,6 @@ proc portclean::clean_dist {args} {
             }
             incr count
         }
-        if {!$usealtworkpath && [file isdirectory ${altprefix}${distdir}]} {
-            ui_debug "Removing directory: ${altprefix}${distdir}"
-            if {[catch {delete ${altprefix}${distdir}} result]} {
-                ui_debug "$::errorInfo"
-                ui_error "$result"
-            }
-            incr count
-        }
     }
     if {$count > 0} {
         ui_debug "$count distfile directory(s) removed."
@@ -205,26 +175,20 @@ proc portclean::clean_dist {args} {
 }
 
 proc portclean::clean_work {args} {
-    global portbuildpath worksymlink usealtworkpath altprefix portpath
+    global portbuildpath subbuildpath worksymlink
 
-    if {[file isdirectory $portbuildpath]} {
-        ui_debug "Removing directory: ${portbuildpath}"
-        if {[catch {delete $portbuildpath} result]} {
+    if {[file isdirectory $subbuildpath]} {
+        ui_debug "Removing directory: ${subbuildpath}"
+        macports_try -pass_signal {
+            delete $subbuildpath
+        } on error {eMessage} {
             ui_debug "$::errorInfo"
-            ui_error "$result"
+            ui_error "$eMessage"
         }
+        # silently fail if non-empty (other subports might be using portbuildpath)
+        catch {file delete $portbuildpath}
     } else {
-        ui_debug "No work directory found to remove at ${portbuildpath}"
-    }
-
-    if {!$usealtworkpath && [file isdirectory ${altprefix}${portbuildpath}]} {
-        ui_debug "Removing directory: ${altprefix}${portbuildpath}"
-        if {[catch {delete ${altprefix}${portbuildpath}} result]} {
-            ui_debug "$::errorInfo"
-            ui_error "$result"
-        }
-    } else {
-        ui_debug "No work directory found to remove at ${altprefix}${portbuildpath}"
+        ui_debug "No work directory found to remove at ${subbuildpath}"
     }
 
     # Clean symlink, if necessary
@@ -232,24 +196,20 @@ proc portclean::clean_work {args} {
         ui_debug "Removing symlink: $worksymlink"
         delete $worksymlink
     }
-    
-    # clean port dir in alt prefix
-    if {[file exists "${altprefix}${portpath}"]} {
-        ui_debug "removing ${altprefix}${portpath}"
-        delete "${altprefix}${portpath}"
-    }
 
     return 0
 }
 proc portclean::clean_logs {args} {
-    global portpath portbuildpath worksymlink name portverbose keeplogs prefix
+    global portpath subport
     set logpath [getportlogpath $portpath]
-  	if {[file isdirectory $logpath]} {
-        ui_debug "Removing directory: ${logpath}"
-        if {[catch {delete $logpath} result]} {
+    set subdir [file join $logpath $subport]
+  	if {[file isdirectory $subdir]} {
+        ui_debug "Removing directory: ${subdir}"
+        if {[catch {delete $subdir} result]} {
             ui_debug "$::errorInfo"
             ui_error "$result"
         }
+        catch {file delete $logpath}
     } else {
         ui_debug "No log directory found to remove at ${logpath}"
     }           	
@@ -257,42 +217,44 @@ proc portclean::clean_logs {args} {
 }
 
 proc portclean::clean_archive {args} {
-    global workpath portarchivepath name version ports_version_glob
+    global subport ports_version_glob portdbpath
 
     # Define archive destination directory, target filename, regex for archive name
-    if {$portarchivepath ne $workpath && $portarchivepath ne ""} {
-        set archivepath [file join $portarchivepath [option os.platform]]
-        set regexstring "^$name-\[\\-_a-zA-Z0-9\\.\]+_\[0-9\]*\[+\\-_a-zA-Z0-9\]*\[\\.\].*\[\\.\]\[a-z2\]+\$"
-    }
+    set archivepath [file join $portdbpath incoming]
 
     if {[info exists ports_version_glob]} {
         # Match all possible archive variants that match the version
-        # glob specified by the user for this OS.
-        set fileglob "$name-[option ports_version_glob]*.*.*"
+        # glob specified by the user.
+        set fileglob "$subport-[option ports_version_glob]*.*.*.*"
     } else {
-        # Match all possible archive variants for the current version on
-        # this OS. If you want to delete previous versions, use the
-        # version glob argument to clean.
-        #
-        # We do this because if we don't, then ports that match the
-        # first part of the name (e.g. trying to remove foo-* will
-        # pick up anything foo-bar-* as well, which is undesirable).
-        set fileglob "$name-$version*.*.*"
+        # Match all possible archives for this port.
+        set fileglob "$subport-*_*.*.*.*"
     }
 
     # Remove the archive files
     set count 0
-    if {![catch {set archivelist [glob [file join $archivepath * $fileglob]]} result]} {
+    foreach dir [list $archivepath ${archivepath}/verified] {
+        set archivelist [glob -nocomplain -directory $dir $fileglob]
         foreach path $archivelist {
-            set file [file tail $path]
-            # Make sure file is truly a port archive file, and not
-            # an accidental match with some other file that might exist.
-            if {[regexp $regexstring $file] && [file isfile $path]} {
+            # Make sure file is truly an archive file for this port, and not
+            # an accidental match with some other file that might exist. Also
+            # delete anything ending in .TMP since those are incomplete and
+            # thus can't be checked and aren't useful anyway.
+            set archivetype [string range [file extension $path] 1 end]
+            if {[file isfile $path] && ($archivetype eq "TMP"
+                || [extract_archive_metadata $path $archivetype portname] eq $subport)} {
                 ui_debug "Removing archive: $path"
                 if {[catch {delete $path} result]} {
                     ui_debug "$::errorInfo"
                     ui_error "$result"
                 }
+                if {[file isfile ${path}.rmd160]} {
+                    ui_debug "Removing archive signature: ${path}.rmd160"
+                    if {[catch {delete ${path}.rmd160} result]} {
+                        ui_debug "$::errorInfo"
+                        ui_error "$result"
+                    }
+                }
                 incr count
             }
         }
@@ -305,4 +267,3 @@ proc portclean::clean_archive {args} {
 
     return 0
 }
-
